/*
 *   Copyright (C) 2014  Alfons Wirtz
 *   website www.freerouting.net
 *
 *   Copyright (C) 2017 Michael Hoffer <info@michaelhoffer.de>
 *   Website www.freerouting.mihosoft.eu
 *
 *   Copyright (C) 2021 Erich S. Heinzle
 *   Website http://www.repo.hu/projects/freerouting_cli/
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License at <http://www.gnu.org/licenses/> 
 *   for more details.
 *
 * MainApplication.java
 *
 * Created on 19. Oktober 2002, 17:58
 *
 */
package eu.mihosoft.freerouting.gui;

import eu.mihosoft.freerouting.board.TestLevel;
import eu.mihosoft.freerouting.constants.Constants;
import eu.mihosoft.freerouting.interactive.InteractiveActionThread;
import eu.mihosoft.freerouting.interactive.ThreadActionListener;
import eu.mihosoft.freerouting.logger.FRLogger;

import eu.mihosoft.freerouting.interactive.BoardHandling;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;

/**
 *
 * Main application for creating frames with new or existing board designs.
 *
 * @author Alfons Wirtz
 */
public class MainApplication
{
    /**
     * Main function of the Application
     * @param args
     */
    public static void main(String[] args)
    {
        FRLogger.traceEntry("MainApplication.main()");

	StartupOptions startupOptions = StartupOptions.parse(args);

	FRLogger.info(startupOptions.testVersion());

        FRLogger.info("freerouting_cli application starting...");

        Thread.setDefaultUncaughtExceptionHandler(new DefaultExceptionHandler());

        if (startupOptions.single_design_option)
        {
            BoardFrame.Option board_option;
            if (startupOptions.session_file_option)
            {
                board_option = BoardFrame.Option.SESSION_FILE;
            }
            else
            {
                board_option = BoardFrame.Option.SINGLE_FRAME;
            }

            FRLogger.info("Opening '"+startupOptions.design_input_filename+"'...");
            DesignFile design_file = DesignFile.get_instance(startupOptions.design_input_filename, false);
            if (design_file == null)
            {
                FRLogger.warn("File with name " +  startupOptions.design_input_filename + " not found");
                return;
            }
            String message = "Loading design " + startupOptions.design_input_filename;

	    FRLogger.info(message);
	    final BoardFrame new_frame =
                    create_board_frame(design_file, board_option,
                            startupOptions.test_version_option, 
                            startupOptions.design_rules_filename);
	    
       	    if (new_frame == null)
                {
                    FRLogger.warn("Couldn't create window frame");
                    System.exit(1);
                    return;
                }
	    new_frame.board_panel.board_handling.settings.autoroute_settings.set_stop_pass_no(startupOptions.max_autoroute_passes);

	    if (startupOptions.pre_route_fanout) {
                new_frame.board_panel.board_handling.settings.autoroute_settings.set_with_fanout(true);
            }

            new_frame.board_panel.board_handling.settings.autoroute_settings.set_with_postroute(startupOptions.max_postroute_passes);

            if (startupOptions.max_autoroute_passes <= 99999)
            {
                InteractiveActionThread thread;
		thread = new_frame.board_panel.board_handling.start_batch_autorouter();
                thread.addListener(new ThreadActionListener() {
                    @Override
                    public void autorouterStarted() {
                    }

                    @Override
                    public void autorouterAborted() {
                        ExportBoardToFile(startupOptions.design_output_filename);
                    }

                    @Override
                    public void autorouterFinished() {
                        ExportBoardToFile(startupOptions.design_output_filename);
                    }

                    private void ExportBoardToFile(String filename) {
                        if ((filename != null)
                                && ((filename.toLowerCase().endsWith(".dsn"))
                                || (filename.toLowerCase().endsWith(".ses"))
                                || (filename.toLowerCase().endsWith(".scr")))) {

                            FRLogger.info("Saving '" + filename + "'...");
                            try {
                                String filename_only = new File(filename).getName();
                                String design_name = filename_only.substring(0, filename_only.length() - 4);

                                java.io.OutputStream output_stream = new java.io.FileOutputStream(filename);

                                if (filename.toLowerCase().endsWith(".dsn")) {
                                    new_frame.board_panel.board_handling.export_to_dsn_file(output_stream, design_name, false);
                                } else if (filename.toLowerCase().endsWith(".ses")) {
                                    new_frame.board_panel.board_handling.export_specctra_session_file(design_name, output_stream);
                                } else if (filename.toLowerCase().endsWith(".scr")) {
					// ditched script export
                                }

                                Runtime.getRuntime().exit(0);
                            } catch (Exception e) {
                                FRLogger.error("Couldn't export board to file", e);
                            }
                        } else {
                            FRLogger.warn("Couldn't export board to '" + filename + "'.");
                        }
                    }
                });
            }
        }
        FRLogger.traceExit("MainApplication.main()");
    }

    /**
     * Creates new form MainApplication
     * It takes the directory of the board designs as optional argument.
     * @param startupOptions
     */
    public MainApplication(StartupOptions startupOptions)
    {
        this.design_dir_name = startupOptions.getDesignDir();
        this.is_test_version = startupOptions.isTestVersion();
    }



    /**
     * Creates a new board frame containing the data of the input design file.
     * Returns null, if an error occurred.
     */
    static private BoardFrame create_board_frame(DesignFile p_design_file, 
            BoardFrame.Option p_option, boolean p_is_test_version, String p_design_rules_file)
    {

        java.io.InputStream input_stream = p_design_file.get_input_stream();
        if (input_stream == null)
        {
	    FRLogger.error("Error: null design file input", null);
            return null;
        }

        TestLevel test_level;
        if (p_is_test_version)
        {
            test_level = TestLevel.TEST_VERSION;
        }
        else
        {
            test_level = TestLevel.RELEASE_VERSION;
        }
        BoardFrame new_frame = new BoardFrame(p_design_file, p_option, test_level, !p_is_test_version);
	///boolean read_ok = new_frame.read(input_stream, p_design_file.is_created_from_text_file(), p_message_field);
	boolean read_ok = new_frame.read(input_stream, p_design_file.is_created_from_text_file());
        if (!read_ok)
        {
            return null;
        }
        if (p_design_file.is_created_from_text_file())
        {
            // Read the file  with the saved rules, if it is existing.

            String file_name = p_design_file.get_name();
            String[] name_parts = file_name.split("\\.");

            String design_name = name_parts[0];

            String parent_folder_name = null;
            String rules_file_name = null;
            String confirm_import_rules_message = null;
            if (p_design_rules_file == null) {
                parent_folder_name = p_design_file.get_parent();
                rules_file_name = design_name + ".rules";
                confirm_import_rules_message = "Please confirm importing stored rules\nBeware that rules imported from the host system may be overwritte";
            } else {
                rules_file_name = p_design_rules_file;
            }

            DesignFile.read_rules_file(design_name, parent_folder_name, rules_file_name,
                    new_frame.board_panel.board_handling, p_option == BoardFrame.Option.WEBSTART,
                    confirm_import_rules_message);
        }
        return new_frame;
    }

    /** The list of open board frames */
    private final java.util.Collection<BoardFrame> board_frames 
            = new java.util.LinkedList<>();
    private String design_dir_name = null;
    private final boolean is_test_version;

    static final String WEB_FILE_BASE_NAME = "http://www.freerouting.mihosoft.eu";

    static final String VERSION_NUMBER_STRING = 
        "v" + Constants.FREEROUTING_VERSION
            + " (build-date: "
            + Constants.FREEROUTING_BUILD_DATE +")";
}
